@@ -1,5 +1,6 @@
 from __future__ import division
 import re, warnings
+import matplotlib
 import matplotlib.cbook as cbook
 from transforms import Bbox, IdentityTransform, TransformedBbox, TransformedPath
 from path import Path
@@ -21,6 +22,38 @@ from path import Path
 # http://groups.google.com/groups?hl=en&lr=&threadm=mailman.5090.1098044946.5135.python-list%40python.org&rnum=1&prev=/groups%3Fq%3D__doc__%2Bauthor%253Ajdhunter%2540ace.bsd.uchicago.edu%26hl%3Den%26btnG%3DGoogle%2BSearch
 
 
+
+
+def allow_rasterization(draw):
+    """
+    Decorator for Artist.draw method. Provides routines
+    that run before and after the draw call. The before and after functions
+    are useful for changing artist-dependant renderer attributes or making
+    other setup function calls, such as starting and flushing a mixed-mode
+    renderer.
+    """
+    def before(artist, renderer):
+        if artist.get_rasterized():
+            renderer.start_rasterizing()
+
+    def after(artist, renderer):
+        if artist.get_rasterized():
+            renderer.stop_rasterizing()
+
+    # the axes class has a second argument inframe for its draw method.
+    def draw_wrapper(artist, renderer, *kl):
+        before(artist, renderer)
+        draw(artist, renderer, *kl)
+        after(artist, renderer)
+
+    # "safe wrapping" to exactly replicate anything we haven't overridden above
+    draw_wrapper.__name__ = draw.__name__
+    draw_wrapper.__dict__ = draw.__dict__
+    draw_wrapper.__doc__  = draw.__doc__
+    draw_wrapper._supports_rasterization = True
+    return draw_wrapper
+
+
 class Artist(object):
     """
     Abstract base class for someone who renders into a
@@ -39,17 +72,23 @@ class Artist(object):
         self._alpha = 1.0
         self.clipbox = None
         self._clippath = None
-        self._clipon = False
+        self._clipon = True
         self._lod = False
         self._label = ''
         self._picker = None
         self._contains = None
+        self._rasterized = None
 
         self.eventson = False  # fire events only if eventson
         self._oid = 0  # an observer id
         self._propobservers = {} # a dict from oids to funcs
         self.axes = None
         self._remove_method = None
+        self._url = None
+        self._gid = None
+        self.x_isdata = True  # False to avoid updating Axes.dataLim with x
+        self.y_isdata = True  #                                      with y
+        self._snap = None
 
     def remove(self):
         """
@@ -81,15 +120,15 @@ class Artist(object):
         # TODO: add legend support
 
     def have_units(self):
-        'return True if units are set on the x or y axes'
+        'Return *True* if units are set on the *x* or *y* axes'
         ax = self.axes
         if ax is None or ax.xaxis is None:
             return False
         return ax.xaxis.have_units() or ax.yaxis.have_units()
 
     def convert_xunits(self, x):
-        """for artists in an axes, if the xaxis as units support,
-        convert x using xaxis unit type
+        """For artists in an axes, if the xaxis has units support,
+        convert *x* using xaxis unit type
         """
         ax = getattr(self, 'axes', None)
         if ax is None or ax.xaxis is None:
@@ -98,8 +137,8 @@ class Artist(object):
         return ax.xaxis.convert_units(x)
 
     def convert_yunits(self, y):
-        """for artists in an axes, if the yaxis as units support,
-        convert y using yaxis unit type
+        """For artists in an axes, if the yaxis has units support,
+        convert *y* using yaxis unit type
         """
         ax = getattr(self, 'axes', None)
         if ax is None or ax.yaxis is None: return y
@@ -107,39 +146,67 @@ class Artist(object):
 
     def set_axes(self, axes):
         """
-        set the axes instance the artist resides in, if any
+        Set the :class:`~matplotlib.axes.Axes` instance in which the
+        artist resides, if any.
 
-        ACCEPTS: an axes instance
+        ACCEPTS: an :class:`~matplotlib.axes.Axes` instance
         """
         self.axes = axes
 
     def get_axes(self):
-        'return the axes instance the artist resides in, or None'
+        """
+        Return the :class:`~matplotlib.axes.Axes` instance the artist
+        resides in, or *None*
+        """
         return self.axes
 
     def add_callback(self, func):
+        """
+        Adds a callback function that will be called whenever one of
+        the :class:`Artist`'s properties changes.
+
+        Returns an *id* that is useful for removing the callback with
+        :meth:`remove_callback` later.
+        """
         oid = self._oid
         self._propobservers[oid] = func
         self._oid += 1
         return oid
 
     def remove_callback(self, oid):
+        """
+        Remove a callback based on its *id*.
+
+        .. seealso::
+
+            :meth:`add_callback`
+               For adding callbacks
+
+        """
         try: del self._propobservers[oid]
         except KeyError: pass
 
     def pchanged(self):
-        'fire event when property changed'
+        """
+        Fire an event when property changed, calling all of the
+        registered callbacks.
+        """
         for oid, func in self._propobservers.items():
             func(self)
 
     def is_transform_set(self):
-        'Artist has transform explicity let'
+        """
+        Returns *True* if :class:`Artist` has a transform explicitly
+        set.
+        """
         return self._transformSet
 
     def set_transform(self, t):
         """
         Set the :class:`~matplotlib.transforms.Transform` instance
         used by this artist.
+
+        ACCEPTS: :class:`~matplotlib.transforms.Transform` instance
         """
         self._transform = t
         self._transformSet = True
@@ -154,8 +221,10 @@ class Artist(object):
             self._transform = IdentityTransform()
         return self._transform
 
-    def hitlist(self,event):
-        """List the children of the artist which contain the mouse event"""
+    def hitlist(self, event):
+        """
+        List the children of the artist which contain the mouse event *event*.
+        """
         import traceback
         L = []
         try:
@@ -166,11 +235,19 @@ class Artist(object):
             traceback.print_exc()
             print "while checking",self.__class__
 
-        if hasattr(self,'get_children'):
-            for a in self.get_children(): L.extend(a.hitlist(event))
+
+        for a in self.get_children():
+            L.extend(a.hitlist(event))
         return L
 
-    def contains(self,mouseevent):
+    def get_children(self):
+        """
+        Return a list of the child :class:`Artist`s this
+        :class:`Artist` contains.
+        """
+        return []
+
+    def contains(self, mouseevent):
         """Test whether the artist contains the mouse event.
 
         Returns the truth value and a dictionary of artist specific details of
@@ -183,32 +260,40 @@ class Artist(object):
         return False,{}
 
     def set_contains(self,picker):
-        """Replace the contains test used by this artist. The new picker should
-        be a callable function which determines whether the artist is hit by the
-        mouse event::
+        """
+        Replace the contains test used by this artist. The new picker
+        should be a callable function which determines whether the
+        artist is hit by the mouse event::
 
             hit, props = picker(artist, mouseevent)
 
-        If the mouse event is over the artist, return hit=True and props
-        is a dictionary of properties you want returned with the contains test.
+        If the mouse event is over the artist, return *hit* = *True*
+        and *props* is a dictionary of properties you want returned
+        with the contains test.
+
+        ACCEPTS: a callable function
         """
         self._contains = picker
 
     def get_contains(self):
-        'return the _contains test used by the artist, or None for default.'
+        """
+        Return the _contains test used by the artist, or *None* for default.
+        """
         return self._contains
 
     def pickable(self):
-        'return True if self is pickable'
+        'Return *True* if :class:`Artist` is pickable.'
         return (self.figure is not None and
                 self.figure.canvas is not None and
                 self._picker is not None)
 
     def pick(self, mouseevent):
         """
-        pick(mouseevent)
+        call signature::
 
-        each child artist will fire a pick event if mouseevent is over
+          pick(mouseevent)
+
+        each child artist will fire a pick event if *mouseevent* is over
         the artist and the artist has picker set
         """
         # Pick self
@@ -222,51 +307,116 @@ class Artist(object):
                 self.figure.canvas.pick_event(mouseevent, self, **prop)
 
         # Pick children
-        if hasattr(self,'get_children'):
-            for a in self.get_children(): a.pick(mouseevent)
+        for a in self.get_children():
+            # make sure the event happened in the same axes
+            ax = getattr(a, 'axes', None)
+            if mouseevent.inaxes==ax:
+                a.pick(mouseevent)
 
     def set_picker(self, picker):
         """
-        set the epsilon for picking used by this artist
+        Set the epsilon for picking used by this artist
 
-        picker can be one of the following:
+        *picker* can be one of the following:
 
-          None -  picking is disabled for this artist (default)
+          * *None*: picking is disabled for this artist (default)
 
-          boolean - if True then picking will be enabled and the
-          artist will fire a pick event if the mouse event is over
-          the artist
+          * A boolean: if *True* then picking will be enabled and the
+            artist will fire a pick event if the mouse event is over
+            the artist
 
-          float - if picker is a number it is interpreted as an
-          epsilon tolerance in points and the the artist will fire
-          off an event if it's data is within epsilon of the mouse
-          event.  For some artists like lines and patch collections,
-          the artist may provide additional data to the pick event
-          that is generated, eg the indices of the data within
-          epsilon of the pick event
+          * A float: if picker is a number it is interpreted as an
+            epsilon tolerance in points and the artist will fire
+            off an event if it's data is within epsilon of the mouse
+            event.  For some artists like lines and patch collections,
+            the artist may provide additional data to the pick event
+            that is generated, e.g. the indices of the data within
+            epsilon of the pick event
 
-          function - if picker is callable, it is a user supplied
-          function which determines whether the artist is hit by the
-          mouse event::
+          * A function: if picker is callable, it is a user supplied
+            function which determines whether the artist is hit by the
+            mouse event::
 
               hit, props = picker(artist, mouseevent)
 
-          to determine the hit test.  if the mouse event is over the
-          artist, return hit=True and props is a dictionary of
-          properties you want added to the PickEvent attributes.
+            to determine the hit test.  if the mouse event is over the
+            artist, return *hit=True* and props is a dictionary of
+            properties you want added to the PickEvent attributes.
 
         ACCEPTS: [None|float|boolean|callable]
         """
         self._picker = picker
 
     def get_picker(self):
-        'return the Pickeration instance used by this artist'
+        'Return the picker object used by this artist'
         return self._picker
 
-
     def is_figure_set(self):
+        """
+        Returns True if the artist is assigned to a
+        :class:`~matplotlib.figure.Figure`.
+        """
         return self.figure is not None
 
+    def get_url(self):
+        """
+        Returns the url
+        """
+        return self._url
+
+    def set_url(self, url):
+        """
+        Sets the url for the artist
+
+        ACCEPTS: a url string
+        """
+        self._url = url
+
+
+    def get_gid(self):
+        """
+        Returns the group id
+        """
+        return self._gid
+
+    def set_gid(self, gid):
+        """
+        Sets the (group) id for the artist
+
+        ACCEPTS: an id string
+        """
+        self._gid = gid
+
+    def get_snap(self):
+        """
+        Returns the snap setting which may be:
+
+          * True: snap vertices to the nearest pixel center
+
+          * False: leave vertices as-is
+
+          * None: (auto) If the path contains only rectilinear line
+            segments, round to the nearest pixel center
+
+        Only supported by the Agg backends.
+        """
+        return self._snap
+
+    def set_snap(self, snap):
+        """
+        Sets the snap setting which may be:
+
+          * True: snap vertices to the nearest pixel center
+
+          * False: leave vertices as-is
+
+          * None: (auto) If the path contains only rectilinear line
+            segments, round to the nearest pixel center
+
+        Only supported by the Agg backends.
+        """
+        self._snap = snap
+
     def get_figure(self):
         """
         Return the :class:`~matplotlib.figure.Figure` instance the
@@ -279,40 +429,40 @@ class Artist(object):
         Set the :class:`~matplotlib.figure.Figure` instance the artist
         belongs to.
 
-        ACCEPTS: a matplotlib.figure.Figure instance
+        ACCEPTS: a :class:`matplotlib.figure.Figure` instance
         """
         self.figure = fig
         self.pchanged()
 
     def set_clip_box(self, clipbox):
         """
-        Set the artist's clip Bbox
+        Set the artist's clip :class:`~matplotlib.transforms.Bbox`.
 
-        ACCEPTS: a matplotlib.transform.Bbox instance
+        ACCEPTS: a :class:`matplotlib.transforms.Bbox` instance
         """
         self.clipbox = clipbox
-        self._clipon = clipbox is not None or self._clippath is not None
         self.pchanged()
 
     def set_clip_path(self, path, transform=None):
         """
         Set the artist's clip path, which may be:
 
-          a) a :class:`~matplotlib.patches.Patch` (or subclass) instance
+          * a :class:`~matplotlib.patches.Patch` (or subclass) instance
 
-          b) a :class:`~matplotlib.path.Path` instance, in which case
+          * a :class:`~matplotlib.path.Path` instance, in which case
              an optional :class:`~matplotlib.transforms.Transform`
              instance may be provided, which will be applied to the
              path before using it for clipping.
 
-          c) *None*, to remove the clipping path
+          * *None*, to remove the clipping path
 
         For efficiency, if the path happens to be an axis-aligned
         rectangle, this method will set the clipping box to the
         corresponding rectangle and set the clipping path to *None*.
 
-        ACCEPTS: a Path instance and a Transform instance, a Patch
-        instance, or None
+        ACCEPTS: [ (:class:`~matplotlib.path.Path`,
+        :class:`~matplotlib.transforms.Transform`) |
+        :class:`~matplotlib.patches.Patch` | None ]
         """
         from patches import Patch, Rectangle
 
@@ -339,7 +489,6 @@ class Artist(object):
             print type(path), type(transform)
             raise TypeError("Invalid arguments to set_clip_path")
 
-        self._clipon = self.clipbox is not None or self._clippath is not None
         self.pchanged()
 
     def get_alpha(self):
@@ -350,16 +499,16 @@ class Artist(object):
         return self._alpha
 
     def get_visible(self):
-        "return the artist's visiblity"
+        "Return the artist's visiblity"
         return self._visible
 
     def get_animated(self):
-        "return the artist's animated state"
+        "Return the artist's animated state"
         return self._animated
 
     def get_clip_on(self):
         'Return whether artist uses clipping'
-        return self._clipon and (self.clipbox is not None or self._clippath is not None)
+        return self._clipon
 
     def get_clip_box(self):
         'Return artist clipbox'
@@ -371,8 +520,9 @@ class Artist(object):
 
     def get_transformed_clip_path_and_affine(self):
         '''
-        Return the clip path with the non-affine part of its transformation applied,
-        and the remaining affine part of its transformation.
+        Return the clip path with the non-affine part of its
+        transformation applied, and the remaining affine part of its
+        transformation.
         '''
         if self._clippath is not None:
             return self._clippath.get_transformed_path_and_affine()
@@ -380,21 +530,38 @@ class Artist(object):
 
     def set_clip_on(self, b):
         """
-        Set  whether artist uses clipping
+        Set whether artist uses clipping.
 
         ACCEPTS: [True | False]
         """
         self._clipon = b
-        if not b:
-            self.clipbox = None
-            self._clippath = None
         self.pchanged()
 
     def _set_gc_clip(self, gc):
-        'set the clip properly for the gc'
-        if self.clipbox is not None:
-            gc.set_clip_rectangle(self.clipbox)
-        gc.set_clip_path(self._clippath)
+        'Set the clip properly for the gc'
+        if self._clipon:
+            if self.clipbox is not None:
+                gc.set_clip_rectangle(self.clipbox)
+            gc.set_clip_path(self._clippath)
+        else:
+            gc.set_clip_rectangle(None)
+            gc.set_clip_path(None)
+
+    def get_rasterized(self):
+        return self._rasterized
+
+    def set_rasterized(self, rasterized):
+        """
+        Force rasterized (bitmap) drawing in vector backend output.
+
+        Defaults to None, which implies the backend's default behavior
+
+        ACCEPTS: [True | False | None]
+        """
+        if rasterized and not hasattr(self.draw, "_supports_rasterization"):
+            warnings.warn("Rasterization of '%s' will be ignored" % self)
+
+        self._rasterized = rasterized
 
     def draw(self, renderer, *args, **kwargs):
         'Derived classes drawing method'
@@ -403,9 +570,9 @@ class Artist(object):
     def set_alpha(self, alpha):
         """
         Set the alpha value used for blending - not supported on
-        all backends
+        all backends.
 
-        ACCEPTS: float
+        ACCEPTS: float (0.0 transparent through 1.0 opaque)
         """
         self._alpha = alpha
         self.pchanged()
@@ -423,7 +590,7 @@ class Artist(object):
 
     def set_visible(self, b):
         """
-        set the artist's visiblity
+        Set the artist's visiblity.
 
         ACCEPTS: [True | False]
         """
@@ -433,7 +600,7 @@ class Artist(object):
 
     def set_animated(self, b):
         """
-        set the artist's animation state
+        Set the artist's animation state.
 
         ACCEPTS: [True | False]
         """
@@ -441,6 +608,10 @@ class Artist(object):
         self.pchanged()
 
     def update(self, props):
+        """
+        Update the properties of this :class:`Artist` from the
+        dictionary *prop*.
+        """
         store = self.eventson
         self.eventson = False
         changed = False
@@ -455,11 +626,14 @@ class Artist(object):
 
 
     def get_label(self):
+        """
+        Get the label used for this artist in the legend.
+        """
         return self._label
 
     def set_label(self, s):
         """
-        Set the line label to s for auto legend
+        Set the label to *s* for auto legend.
 
         ACCEPTS: any string
         """
@@ -468,11 +642,16 @@ class Artist(object):
 
 
 
-    def get_zorder(self): return self.zorder
+    def get_zorder(self):
+        """
+        Return the :class:`Artist`'s zorder.
+        """
+        return self.zorder
 
     def set_zorder(self, level):
         """
-        Set the zorder for the artist
+        Set the zorder for the artist.  Artists with lower zorder
+        values are drawn first.
 
         ACCEPTS: any number
         """
@@ -493,9 +672,15 @@ class Artist(object):
         self.pchanged()
 
 
+    def properties(self):
+        """
+        return a dictionary mapping property name -> value for all Artist props
+        """
+        return ArtistInspector(self).properties()
+
     def set(self, **kwargs):
         """
-        A tkstyle set command, pass kwargs to set properties
+        A tkstyle set command, pass *kwargs* to set properties
         """
         ret = []
         for k,v in kwargs.items():
@@ -505,6 +690,51 @@ class Artist(object):
             ret.extend( [func(v)] )
         return ret
 
+    def findobj(self, match=None):
+        """
+        pyplot signature:
+          findobj(o=gcf(), match=None)
+
+        Recursively find all :class:matplotlib.artist.Artist instances
+        contained in self.
+
+        *match* can be
+
+          - None: return all objects contained in artist (including artist)
+
+          - function with signature ``boolean = match(artist)`` used to filter matches
+
+          - class instance: eg Line2D.  Only return artists of class type
+
+        .. plot:: mpl_examples/pylab_examples/findobj_demo.py
+        """
+
+        if match is None: # always return True
+            def matchfunc(x): return True
+        elif cbook.issubclass_safe(match, Artist):
+            def matchfunc(x):
+                return isinstance(x, match)
+        elif callable(match):
+            matchfunc = match
+        else:
+            raise ValueError('match must be None, an matplotlib.artist.Artist subclass, or a callable')
+
+
+        artists = []
+
+        for c in self.get_children():
+            if matchfunc(c):
+                artists.append(c)
+            artists.extend([thisc for thisc in c.findobj(matchfunc) if matchfunc(thisc)])
+
+        if matchfunc(self):
+            artists.append(self)
+        return artists
+
+
+
+
+
 
 class ArtistInspector:
     """
@@ -521,13 +751,18 @@ class ArtistInspector:
         type) and it is your responsibility to make sure this is so.
         """
         if cbook.iterable(o) and len(o): o = o[0]
+
+        self.oorig = o
+        if not isinstance(o, type):
+            o = type(o)
         self.o = o
+
         self.aliasd = self.get_aliases()
 
     def get_aliases(self):
         """
-        Get a dict mapping *fullname* -> *alias* for each alias in the
-        :class:`~matplotlib.artist.ArtistInspector`.
+        Get a dict mapping *fullname* -> *alias* for each *alias* in
+        the :class:`~matplotlib.artist.ArtistInspector`.
 
         Eg., for lines::
 
@@ -545,15 +780,15 @@ class ArtistInspector:
             if not self.is_alias(func): continue
             docstring = func.__doc__
             fullname = docstring[10:]
-            aliases[fullname[4:]] = name[4:]
+            aliases.setdefault(fullname[4:], {})[name[4:]] = None
         return aliases
 
-    _get_valid_values_regex = re.compile(r"\n\s*ACCEPTS:\s*(.*)\n")
+    _get_valid_values_regex = re.compile(r"\n\s*ACCEPTS:\s*((?:.|\n)*?)(?:$|(?:\n\n))")
     def get_valid_values(self, attr):
         """
         Get the legal arguments for the setter associated with *attr*.
 
-        This is done by querying the docstring of the function set_ *attr*
+        This is done by querying the docstring of the function *set_attr*
         for a line that begins with ACCEPTS:
 
         Eg., for a line linestyle, return
@@ -573,24 +808,37 @@ class ArtistInspector:
 
         match = self._get_valid_values_regex.search(docstring)
         if match is not None:
-            return match.group(1)
+            return match.group(1).replace('\n', ' ')
         return 'unknown'
 
+    def _get_setters_and_targets(self):
+        """
+        Get the attribute strings and a full path to where the setter
+        is defined for all setters in an object.
+        """
+
+        setters = []
+        for name in dir(self.o):
+            if not name.startswith('set_'): continue
+            o = getattr(self.o, name)
+            if not callable(o): continue
+            func = o
+            if self.is_alias(func): continue
+            source_class = self.o.__module__ + "." + self.o.__name__
+            for cls in self.o.mro():
+                if name in cls.__dict__:
+                    source_class = cls.__module__ + "." + cls.__name__
+                    break
+            setters.append((name[4:], source_class + "." + name))
+        return setters
+
     def get_setters(self):
         """
         Get the attribute strings with setters for object.  Eg., for a line,
         return ``['markerfacecolor', 'linewidth', ....]``.
         """
 
-        setters = []
-        for name in dir(self.o):
-            if not name.startswith('set_'): continue
-            o = getattr(self.o,name)
-            if not callable(o): continue
-            func = o
-            if self.is_alias(func): continue
-            setters.append(name[4:])
-        return setters
+        return [prop for prop, target in self._get_setters_and_targets()]
 
     def is_alias(self, o):
         """
@@ -603,16 +851,37 @@ class ArtistInspector:
 
     def aliased_name(self, s):
         """
-        return 'PROPNAME or alias' if s has an alias, else return
+        return 'PROPNAME or alias' if *s* has an alias, else return
         PROPNAME.
 
-        Eg for the line markerfacecolor property, which has an alias,
-        return 'markerfacecolor or mfc' and for the transform
+        E.g. for the line markerfacecolor property, which has an
+        alias, return 'markerfacecolor or mfc' and for the transform
         property, which does not, return 'transform'
         """
-        if self.aliasd.has_key(s):
-            return '%s or %s' % (s, self.aliasd[s])
-        else: return s
+
+        if s in self.aliasd:
+            return s + ''.join([' or %s' % x for x in self.aliasd[s].keys()])
+        else:
+            return s
+
+
+    def aliased_name_rest(self, s, target):
+        """
+        return 'PROPNAME or alias' if *s* has an alias, else return
+        PROPNAME formatted for ReST
+
+        E.g. for the line markerfacecolor property, which has an
+        alias, return 'markerfacecolor or mfc' and for the transform
+        property, which does not, return 'transform'
+        """
+
+        if s in self.aliasd:
+            aliases = ''.join([' or %s' % x for x in self.aliasd[s].keys()])
+        else:
+            aliases = ''
+        return ':meth:`%s <%s>`%s' % (s, target, aliases)
+
+
 
     def pprint_setters(self, prop=None, leadingspace=2):
         """
@@ -631,13 +900,41 @@ class ArtistInspector:
             accepts = self.get_valid_values(prop)
             return '%s%s: %s' %(pad, prop, accepts)
 
-        attrs = self.get_setters()
+        attrs = self._get_setters_and_targets()
+        attrs.sort()
+        lines = []
+
+        for prop, path in attrs:
+            accepts = self.get_valid_values(prop)
+            name = self.aliased_name(prop)
+
+            lines.append('%s%s: %s' %(pad, name, accepts))
+        return lines
+
+    def pprint_setters_rest(self, prop=None, leadingspace=2):
+        """
+        If *prop* is *None*, return a list of strings of all settable properies
+        and their valid values.  Format the output for ReST
+
+        If *prop* is not *None*, it is a valid property name and that
+        property will be returned as a string of property : valid
+        values.
+        """
+        if leadingspace:
+            pad = ' '*leadingspace
+        else:
+            pad  = ''
+        if prop is not None:
+            accepts = self.get_valid_values(prop)
+            return '%s%s: %s' %(pad, prop, accepts)
+
+        attrs = self._get_setters_and_targets()
         attrs.sort()
         lines = []
 
         ########
-        names = [self.aliased_name(prop) for prop in attrs]
-        accepts = [self.get_valid_values(prop) for prop in attrs]
+        names = [self.aliased_name_rest(prop, target) for prop, target in attrs]
+        accepts = [self.get_valid_values(prop) for prop, target in attrs]
 
         col0_len = max([len(n) for n in names])
         col1_len = max([len(a) for a in accepts])
@@ -657,73 +954,142 @@ class ArtistInspector:
         return lines
         ########
 
-        for prop in attrs:
+        for prop, path in attrs:
             accepts = self.get_valid_values(prop)
-            name = self.aliased_name(prop)
+            name = self.aliased_name_rest(prop, path)
 
             lines.append('%s%s: %s' %(pad, name, accepts))
         return lines
 
+
+    def properties(self):
+        """
+        return a dictionary mapping property name -> value
+        """
+        o = self.oorig
+        getters = [name for name in dir(o)
+                   if name.startswith('get_')
+                   and callable(getattr(o, name))]
+        #print getters
+        getters.sort()
+        d = dict()
+        for name in getters:
+            func = getattr(o, name)
+            if self.is_alias(func): continue
+
+            try: val = func()
+            except: continue
+            else: d[name[4:]] = val
+
+        return d
+
     def pprint_getters(self):
         """
         Return the getters and actual values as list of strings.
         """
-        getters = [name for name in dir(self.o)
-                   if name.startswith('get_')
-                   and callable(getattr(self.o, name))]
-        getters.sort()
+
+        d = self.properties()
+        names = d.keys()
+        names.sort()
         lines = []
-        for name in getters:
-            func = getattr(self.o, name)
-            if self.is_alias(func): continue
-            try: val = func()
-            except: continue
+        for name in names:
+            val = d[name]
             if getattr(val, 'shape', ()) != () and len(val)>6:
                 s = str(val[:6]) + '...'
             else:
                 s = str(val)
+            s = s.replace('\n', ' ')
+            if len(s)>50:
+                s = s[:50] + '...'
             name = self.aliased_name(name[4:])
             lines.append('    %s = %s' %(name, s))
         return lines
 
 
-def getp(o, *args):
+
+    def findobj(self, match=None):
+        """
+        Recursively find all :class:`matplotlib.artist.Artist`
+        instances contained in *self*.
+
+        If *match* is not None, it can be
+
+          - function with signature ``boolean = match(artist)``
+
+          - class instance: eg :class:`~matplotlib.lines.Line2D`
+
+        used to filter matches.
+        """
+
+        if match is None: # always return True
+            def matchfunc(x): return True
+        elif issubclass(match, Artist):
+            def matchfunc(x):
+                return isinstance(x, match)
+        elif callable(match):
+            matchfunc = func
+        else:
+            raise ValueError('match must be None, an matplotlib.artist.Artist subclass, or a callable')
+
+
+        artists = []
+
+        for c in self.get_children():
+            if matchfunc(c):
+                artists.append(c)
+            artists.extend([thisc for thisc in c.findobj(matchfunc) if matchfunc(thisc)])
+
+        if matchfunc(self):
+            artists.append(self)
+        return artists
+
+
+
+
+
+
+def getp(o, property=None):
     """
-    .. TODO: What are 's' and 'h' arguments described below?
+    Return the value of handle property.  property is an optional string
+    for the property you want to return
 
-    Return the value of handle property s
+    Example usage::
 
-    h is an instance of a class, eg a Line2D or an Axes or Text.
-    if s is 'somename', this function returns
+        getp(o)  # get all the object properties
+        getp(o, 'linestyle')  # get the linestyle property
+
+    *o* is a :class:`Artist` instance, eg
+    :class:`~matplotllib.lines.Line2D` or an instance of a
+    :class:`~matplotlib.axes.Axes` or :class:`matplotlib.text.Text`.
+    If the *property* is 'somename', this function returns
 
       o.get_somename()
 
-    getp can be used to query all the gettable properties with getp(o)
-    Many properties have aliases for shorter typing, eg 'lw' is an
-    alias for 'linewidth'.  In the output, aliases and full property
-    names will be listed as
+    :func:`getp` can be used to query all the gettable properties with
+    ``getp(o)``. Many properties have aliases for shorter typing, e.g.
+    'lw' is an alias for 'linewidth'.  In the output, aliases and full
+    property names will be listed as:
 
-      property or  alias = value
+      property or alias = value
 
-    eg
+    e.g.:
 
       linewidth or lw = 2
     """
 
     insp = ArtistInspector(o)
 
-    if len(args)==0:
-        print '\n'.join(insp.pprint_getters())
+    if property is None:
+        ret = insp.pprint_getters()
+        print '\n'.join(ret)
         return
 
-    name = args[0]
-    func = getattr(o, 'get_' + name)
+    func = getattr(o, 'get_' + property)
+
     return func()
 
-def get(o, *args, **kwargs):
-    return getp(o, *args, **kwargs)
-get.__doc__ = getp.__doc__
-
+# alias
+get = getp
 
 def setp(h, *args, **kwargs):
     """
@@ -750,7 +1116,7 @@ def setp(h, *args, **kwargs):
     :func:`setp` operates on a single instance or a list of instances.
     If you are in query mode introspecting the possible values, only
     the first instance in the sequence is used.  When actually setting
-    values, all the instances will be set.  Eg., suppose you have a
+    values, all the instances will be set.  E.g., suppose you have a
     list of two lines, the following will make both lines thicker and
     red::
 
@@ -761,7 +1127,7 @@ def setp(h, *args, **kwargs):
       >>> setp(lines, linewidth=2, color='r')
 
     :func:`setp` works with the matlab(TM) style string/value pairs or
-    with python kwargs.  For example, the following are equivalent
+    with python kwargs.  For example, the following are equivalent::
 
       >>> setp(lines, 'linewidth', 2, 'color', r')  # matlab style
 
@@ -800,7 +1166,11 @@ def setp(h, *args, **kwargs):
     return [x for x in cbook.flatten(ret)]
 
 def kwdoc(a):
-    return '\n'.join(ArtistInspector(a).pprint_setters(leadingspace=2))
+    hardcopy = matplotlib.rcParams['docstring.hardcopy']
+    if hardcopy:
+        return '\n'.join(ArtistInspector(a).pprint_setters_rest(leadingspace=2))
+    else:
+        return '\n'.join(ArtistInspector(a).pprint_setters(leadingspace=2))
 
 kwdocd = dict()
 kwdocd['Artist'] = kwdoc(Artist)
