@@ -8,16 +8,19 @@ counterparts (e.g. you may not be able to select all line styles) but
 they are meant to be fast for common use cases (e.g. a bunch of solid
 line segemnts)
 """
-import math, warnings
+import copy, math, warnings
 import numpy as np
+from numpy import ma
 import matplotlib as mpl
 import matplotlib.cbook as cbook
 import matplotlib.colors as _colors # avoid conflict with kwarg
 import matplotlib.cm as cm
 import matplotlib.transforms as transforms
 import matplotlib.artist as artist
+from matplotlib.artist import allow_rasterization
 import matplotlib.backend_bases as backend_bases
 import matplotlib.path as mpath
+import matplotlib.mlab as mlab
 
 class Collection(artist.Artist, cm.ScalarMappable):
     """
@@ -70,6 +73,7 @@ class Collection(artist.Artist, cm.ScalarMappable):
                  norm = None,  # optional for ScalarMappable
                  cmap = None,  # ditto
                  pickradius = 5.0,
+                 urls = None,
                  **kwargs
                  ):
         """
@@ -85,15 +89,16 @@ class Collection(artist.Artist, cm.ScalarMappable):
         self.set_linewidth(linewidths)
         self.set_linestyle(linestyles)
         self.set_antialiased(antialiaseds)
+        self.set_urls(urls)
+
 
         self._uniform_offsets = None
         self._offsets = np.array([], np.float_)
         if offsets is not None:
-            offsets = np.asarray(offsets, np.float_)
+            offsets = np.asarray(offsets)
             if len(offsets.shape) == 1:
                 offsets = offsets[np.newaxis,:]  # Make it Nx2.
             if transOffset is not None:
-                Affine2D = transforms.Affine2D
                 self._offsets = offsets
                 self._transOffset = transOffset
             else:
@@ -102,12 +107,14 @@ class Collection(artist.Artist, cm.ScalarMappable):
         self._pickradius = pickradius
         self.update(kwargs)
 
+
     def _get_value(self, val):
         try: return (float(val), )
         except TypeError:
             if cbook.iterable(val) and len(val):
                 try: float(val[0])
                 except TypeError: pass # raise below
+                except ValueError: pass
                 else: return val
 
         raise TypeError('val must be a float or nonzero sequence of floats')
@@ -148,9 +155,15 @@ class Collection(artist.Artist, cm.ScalarMappable):
         result = result.inverse_transformed(transData)
         return result
 
-    def draw(self, renderer):
-        if not self.get_visible(): return
-        renderer.open_group(self.__class__.__name__)
+    def get_window_extent(self, renderer):
+        bbox = self.get_datalim(transforms.IdentityTransform())
+        #TODO:check to ensure that this does not fail for
+        #cases other than scatter plot legend
+        return bbox
+
+    def _prepare_points(self):
+        """Point prep for drawing and hit testing"""
+
         transform = self.get_transform()
         transOffset = self._transOffset
         offsets = self._offsets
@@ -165,18 +178,12 @@ class Collection(artist.Artist, cm.ScalarMappable):
                 ys = self.convert_yunits(ys)
                 paths.append(mpath.Path(zip(xs, ys), path.codes))
             if len(self._offsets):
-                xs = self.convert_xunits(self._offsets[:0])
-                ys = self.convert_yunits(self._offsets[:1])
+                xs = self.convert_xunits(self._offsets[:,0])
+                ys = self.convert_yunits(self._offsets[:,1])
                 offsets = zip(xs, ys)
 
         offsets = np.asarray(offsets, np.float_)
 
-        self.update_scalarmappable()
-
-        clippath, clippath_trans = self.get_transformed_clip_path_and_affine()
-        if clippath_trans is not None:
-            clippath_trans = clippath_trans.frozen()
-
         if not transform.is_affine:
             paths = [transform.transform_path_non_affine(path) for path in paths]
             transform = transform.get_affine()
@@ -184,12 +191,27 @@ class Collection(artist.Artist, cm.ScalarMappable):
             offsets = transOffset.transform_non_affine(offsets)
             transOffset = transOffset.get_affine()
 
+        return transform, transOffset, offsets, paths
+
+    @allow_rasterization
+    def draw(self, renderer):
+        if not self.get_visible(): return
+        renderer.open_group(self.__class__.__name__)
+
+        self.update_scalarmappable()
+
+        clippath, clippath_trans = self.get_transformed_clip_path_and_affine()
+        if clippath_trans is not None:
+            clippath_trans = clippath_trans.frozen()
+
+        transform, transOffset, offsets, paths = self._prepare_points()
+
         renderer.draw_path_collection(
             transform.frozen(), self.clipbox, clippath, clippath_trans,
             paths, self.get_transforms(),
             offsets, transOffset,
-            self._facecolors, self._edgecolors, self._linewidths,
-            self._linestyles, self._antialiaseds)
+            self.get_facecolor(), self.get_edgecolor(), self._linewidths,
+            self._linestyles, self._antialiaseds, self._urls)
         renderer.close_group(self.__class__.__name__)
 
     def contains(self, mouseevent):
@@ -200,24 +222,54 @@ class Collection(artist.Artist, cm.ScalarMappable):
         item in itemlist contains the event.
         """
         if callable(self._contains): return self._contains(self,mouseevent)
+        if not self.get_visible(): return False,{}
 
-        transform = self.get_transform()
-        paths = self.get_paths()
-        if not transform.is_affine:
-            paths = [transform.transform_path_non_affine(path) for path in paths]
-            transform = transform.get_affine()
+        transform, transOffset, offsets, paths = self._prepare_points()
 
         ind = mpath.point_in_path_collection(
             mouseevent.x, mouseevent.y, self._pickradius,
             transform.frozen(), paths, self.get_transforms(),
-            np.asarray(self._offsets, np.float_),
-            self._transOffset.frozen(), len(self._facecolors))
+            offsets, transOffset, len(self._facecolors)>0)
         return len(ind)>0,dict(ind=ind)
 
     def set_pickradius(self,pickradius): self.pickradius = 5
     def get_pickradius(self): return self.pickradius
 
-    def set_linewidths(self, lw):
+    def set_urls(self, urls):
+        if urls is None:
+            self._urls = [None,]
+        else:
+            self._urls = urls
+
+    def get_urls(self): return self._urls
+
+    def set_offsets(self, offsets):
+        """
+        Set the offsets for the collection.  *offsets* can be a scalar
+        or a sequence.
+
+        ACCEPTS: float or sequence of floats
+        """
+        offsets = np.asarray(offsets, np.float_)
+        if len(offsets.shape) == 1:
+            offsets = offsets[np.newaxis,:]  # Make it Nx2.
+        #This decision is based on how they are initialized above
+        if self._uniform_offsets is None:
+            self._offsets = offsets
+        else:
+            self._uniform_offsets = offsets
+
+    def get_offsets(self):
+        """
+        Return the offsets for the collection.
+        """
+        #This decision is based on how they are initialized above in __init__()
+        if self._uniform_offsets is None:
+            return self._offsets
+        else:
+            return self._uniform_offsets
+
+    def set_linewidth(self, lw):
         """
         Set the linewidth(s) for the collection.  *lw* can be a scalar
         or a sequence; if it is a sequence the patches will cycle
@@ -227,19 +279,28 @@ class Collection(artist.Artist, cm.ScalarMappable):
         """
         if lw is None: lw = mpl.rcParams['patch.linewidth']
         self._linewidths = self._get_value(lw)
-    set_lw = set_linewidth = set_linewidths
 
-    def set_linestyles(self, ls):
+    def set_linewidths(self, lw):
+        """alias for set_linewidth"""
+        return self.set_linewidth(lw)
+
+    def set_lw(self, lw):
+        """alias for set_linewidth"""
+        return self.set_linewidth(lw)
+
+    def set_linestyle(self, ls):
         """
-        Set the linestyles(s) for the collection.
-        ACCEPTS: ['solid' | 'dashed', 'dashdot', 'dotted' | (offset, on-off-dash-seq) ]
+        Set the linestyle(s) for the collection.
+
+        ACCEPTS: ['solid' | 'dashed', 'dashdot', 'dotted' |
+        (offset, on-off-dash-seq) ]
         """
         try:
             dashd = backend_bases.GraphicsContextBase.dashd
             if cbook.is_string_like(ls):
-                if dashd.has_key(ls):
+                if ls in dashd:
                     dashes = [dashd[ls]]
-                elif cbook.ls_mapper.has_key(ls):
+                elif ls in cbook.ls_mapper:
                     dashes = [dashd[cbook.ls_mapper[ls]]]
                 else:
                     raise ValueError()
@@ -248,13 +309,13 @@ class Collection(artist.Artist, cm.ScalarMappable):
                     dashes = []
                     for x in ls:
                         if cbook.is_string_like(x):
-                            if dashd.has_key(x):
+                            if x in dashd:
                                 dashes.append(dashd[x])
-                            elif cbook.ls_mapper.has_key(x):
+                            elif x in cbook.ls_mapper:
                                 dashes.append(dashd[cbook.ls_mapper[x]])
                             else:
                                 raise ValueError()
-                        elif cbook.iterator(x) and len(x) == 2:
+                        elif cbook.iterable(x) and len(x) == 2:
                             dashes.append(x)
                         else:
                             raise ValueError()
@@ -268,7 +329,14 @@ class Collection(artist.Artist, cm.ScalarMappable):
         except ValueError:
             raise ValueError('Do not know how to convert %s to dashes'%ls)
         self._linestyles = dashes
-    set_dashes = set_linestyle = set_linestyles
+
+    def set_linestyles(self, ls):
+        """alias for set_linestyle"""
+        return self.set_linestyle(ls)
+
+    def set_dashes(self, ls):
+        """alias for set_linestyle"""
+        return self.set_linestyle(ls)
 
     def set_antialiased(self, aa):
         """
@@ -279,14 +347,21 @@ class Collection(artist.Artist, cm.ScalarMappable):
         if aa is None:
             aa = mpl.rcParams['patch.antialiased']
         self._antialiaseds = self._get_bool(aa)
-    set_antialiaseds = set_antialiased
+
+    def set_antialiaseds(self, aa):
+        """alias for set_antialiased"""
+        return self.set_antialiased(aa)
 
     def set_color(self, c):
         """
         Set both the edgecolor and the facecolor.
-        See :meth:`set_facecolor` and :meth:`set_edgecolor`.
 
         ACCEPTS: matplotlib color arg or sequence of rgba tuples
+
+        .. seealso::
+
+            :meth:`set_facecolor`, :meth:`set_edgecolor`
+               For setting the edge or face color individually.
         """
         self.set_facecolor(c)
         self.set_edgecolor(c)
@@ -295,37 +370,69 @@ class Collection(artist.Artist, cm.ScalarMappable):
         """
         Set the facecolor(s) of the collection.  *c* can be a
         matplotlib color arg (all patches have same color), or a
-        sequence or rgba tuples; if it is a sequence the patches will
-        cycle through the sequence
+        sequence of rgba tuples; if it is a sequence the patches will
+        cycle through the sequence.
+
+        If *c* is 'none', the patch will not be filled.
 
         ACCEPTS: matplotlib color arg or sequence of rgba tuples
         """
+        self._is_filled = True
+        try:
+            if c.lower() == 'none':
+                self._is_filled = False
+        except AttributeError:
+            pass
         if c is None: c = mpl.rcParams['patch.facecolor']
+        self._facecolors_original = c
         self._facecolors = _colors.colorConverter.to_rgba_array(c, self._alpha)
 
-    set_facecolors = set_facecolor
+    def set_facecolors(self, c):
+        """alias for set_facecolor"""
+        return self.set_facecolor(c)
 
     def get_facecolor(self):
         return self._facecolors
     get_facecolors = get_facecolor
 
     def get_edgecolor(self):
-        return self._edgecolors
+        if self._edgecolors == 'face':
+            return self.get_facecolors()
+        else:
+            return self._edgecolors
     get_edgecolors = get_edgecolor
 
     def set_edgecolor(self, c):
         """
         Set the edgecolor(s) of the collection. *c* can be a
         matplotlib color arg (all patches have same color), or a
-        sequence or rgba tuples; if it is a sequence the patches will
-        cycle through the sequence
+        sequence of rgba tuples; if it is a sequence the patches will
+        cycle through the sequence.
+
+        If *c* is 'face', the edge color will always be the same as
+        the face color.  If it is 'none', the patch boundary will not
+        be drawn.
 
         ACCEPTS: matplotlib color arg or sequence of rgba tuples
         """
-        if c is None: c = mpl.rcParams['patch.edgecolor']
-        self._edgecolors = _colors.colorConverter.to_rgba_array(c, self._alpha)
+        self._is_stroked = True
+        try:
+            if c.lower() == 'none':
+                self._is_stroked = False
+        except AttributeError:
+            pass
+        if c == 'face':
+            self._edgecolors = 'face'
+            self._edgecolors_original = 'face'
+        else:
+            if c is None: c = mpl.rcParams['patch.edgecolor']
+            self._edgecolors_original = c
+            self._edgecolors = _colors.colorConverter.to_rgba_array(c, self._alpha)
 
-    set_edgecolors = set_edgecolor
+
+    def set_edgecolors(self, c):
+        """alias for set_edgecolor"""
+        return self.set_edgecolor(c)
 
     def set_alpha(self, alpha):
         """
@@ -339,11 +446,14 @@ class Collection(artist.Artist, cm.ScalarMappable):
         else:
             artist.Artist.set_alpha(self, alpha)
             try:
-                self._facecolors[:, 3] = alpha
+                self._facecolors = _colors.colorConverter.to_rgba_array(
+                    self._facecolors_original, self._alpha)
             except (AttributeError, TypeError, IndexError):
                 pass
             try:
-                self._edgecolors[:, 3] = alpha
+                if self._edgecolors_original != 'face':
+                    self._edgecolors = _colors.colorConverter.to_rgba_array(
+                        self._edgecolors_original, self._alpha)
             except (AttributeError, TypeError, IndexError):
                 pass
 
@@ -363,11 +473,23 @@ class Collection(artist.Artist, cm.ScalarMappable):
         if self._A is None: return
         if self._A.ndim > 1:
             raise ValueError('Collections can only map rank 1 arrays')
-        if len(self._facecolors):
+        if self._is_filled:
             self._facecolors = self.to_rgba(self._A, self._alpha)
-        else:
+        elif self._is_stroked:
             self._edgecolors = self.to_rgba(self._A, self._alpha)
 
+    def update_from(self, other):
+        'copy properties from other to self'
+
+        artist.Artist.update_from(self, other)
+        self._antialiaseds = other._antialiaseds
+        self._edgecolors_original = other._edgecolors_original
+        self._edgecolors = other._edgecolors
+        self._facecolors_original = other._facecolors_original
+        self._facecolors = other._facecolors
+        self._linewidths = other._linewidths
+        self._linestyles = other._linestyles
+        self._pickradius = other._pickradius
 
 # these are not available for the object inspector until after the
 # class is built so we define an initial set here for the init
@@ -447,7 +569,7 @@ class QuadMesh(Collection):
                 self._meshWidth, self._meshHeight, self._coordinates)
         return self._paths
 
-    #@staticmethod
+    @staticmethod
     def convert_mesh_to_paths(meshWidth, meshHeight, coordinates):
         """
         Converts a given mesh into a sequence of
@@ -458,12 +580,10 @@ class QuadMesh(Collection):
         """
         Path = mpath.Path
 
-        c = coordinates
-        # We could let the Path constructor generate the codes for us,
-        # but this is faster, since we know they'll always be the same
-        codes = np.array(
-            [Path.MOVETO, Path.LINETO, Path.LINETO, Path.LINETO, Path.CLOSEPOLY],
-            Path.code_type)
+        if ma.isMaskedArray(coordinates):
+            c = coordinates.data
+        else:
+            c = coordinates
 
         points = np.concatenate((
                     c[0:-1, 0:-1],
@@ -473,12 +593,12 @@ class QuadMesh(Collection):
                     c[0:-1, 0:-1]
                     ), axis=2)
         points = points.reshape((meshWidth * meshHeight, 5, 2))
-        return [Path(x, codes) for x in points]
-    convert_mesh_to_paths = staticmethod(convert_mesh_to_paths)
+        return [Path(x) for x in points]
 
     def get_datalim(self, transData):
         return self._bbox
 
+    @allow_rasterization
     def draw(self, renderer):
         if not self.get_visible(): return
         renderer.open_group(self.__class__.__name__)
@@ -501,15 +621,25 @@ class QuadMesh(Collection):
         if clippath_trans is not None:
             clippath_trans = clippath_trans.frozen()
 
-        assert transform.is_affine
+        if not transform.is_affine:
+            coordinates = self._coordinates.reshape(
+                (self._coordinates.shape[0] *
+                 self._coordinates.shape[1],
+                 2))
+            coordinates = transform.transform(coordinates)
+            coordinates = coordinates.reshape(self._coordinates.shape)
+            transform = transforms.IdentityTransform()
+        else:
+            coordinates = self._coordinates
+
         if not transOffset.is_affine:
             offsets = transOffset.transform_non_affine(offsets)
             transOffset = transOffset.get_affine()
 
         renderer.draw_quad_mesh(
             transform.frozen(), self.clipbox, clippath, clippath_trans,
-            self._meshWidth, self._meshHeight, self._coordinates,
-            offsets, transOffset, self._facecolors, self._antialiased,
+            self._meshWidth, self._meshHeight, coordinates,
+            offsets, transOffset, self.get_facecolor(), self._antialiased,
             self._showedges)
         renderer.close_group(self.__class__.__name__)
 
@@ -520,8 +650,14 @@ class PolyCollection(Collection):
         *verts_i* is a sequence of *xy* tuples of vertices, or an
         equivalent :mod:`numpy` array of shape (*nv*, 2).
 
-        *sizes* gives the area of the circle circumscribing the
-        polygon in points^2.
+        *sizes* is *None* (default) or a sequence of floats that
+        scale the corresponding *verts_i*.  The scaling is applied
+        before the Artist master transform; if the latter is an identity
+        transform, then the overall scaling is such that if
+        *verts_i* specify a unit square, then *sizes_i* is the area
+        of that square in points^2.
+        If len(*sizes*) < *nv*, the additional values will be
+        taken cyclically from the array.
 
         *closed*, when *True*, will explicitly close the polygon.
 
@@ -534,12 +670,19 @@ class PolyCollection(Collection):
 
     def set_verts(self, verts, closed=True):
         '''This allows one to delay initialization of the vertices.'''
+        if np.ma.isMaskedArray(verts):
+            verts = verts.astype(np.float_).filled(np.nan)
+            # This is much faster than having Path do it one at a time.
         if closed:
             self._paths = []
             for xy in verts:
-                xy = np.asarray(xy)
-                if len(xy) and (xy[0] != xy[-1]).any():
-                    xy = np.concatenate([xy, [xy[0]]])
+                if np.ma.isMaskedArray(xy):
+                    if len(xy) and (xy[0] != xy[-1]).any():
+                        xy = np.ma.concatenate([xy, [xy[0]]])
+                else:
+                    xy = np.asarray(xy)
+                    if len(xy) and (xy[0] != xy[-1]).any():
+                        xy = np.concatenate([xy, [xy[0]]])
                 self._paths.append(mpath.Path(xy))
         else:
             self._paths = [mpath.Path(xy) for xy in verts]
@@ -548,15 +691,14 @@ class PolyCollection(Collection):
         return self._paths
 
     def draw(self, renderer):
-        # sizes is the area of the circle circumscribing the polygon
-        # in points^2
         if self._sizes is not None:
             self._transforms = [
                 transforms.Affine2D().scale(
-                    (np.sqrt(x) * renderer.dpi / 72.0))
+                    (np.sqrt(x) * self.figure.dpi / 72.0))
                 for x in self._sizes]
         return Collection.draw(self, renderer)
 
+
 class BrokenBarHCollection(PolyCollection):
     """
     A collection of horizontal bars spanning *yrange* with a sequence of
@@ -578,6 +720,27 @@ class BrokenBarHCollection(PolyCollection):
         PolyCollection.__init__(self, verts, **kwargs)
     __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
 
+
+    @staticmethod
+    def span_where(x, ymin, ymax, where, **kwargs):
+        """
+        Create a BrokenBarHCollection to plot horizontal bars from
+        over the regions in *x* where *where* is True.  The bars range
+        on the y-axis from *ymin* to *ymax*
+
+        A :class:`BrokenBarHCollection` is returned.  *kwargs* are
+        passed on to the collection.
+        """
+        xranges = []
+        for ind0, ind1 in mlab.contiguous_regions(where):
+            xslice = x[ind0:ind1]
+            if not len(xslice):
+                continue
+            xranges.append((xslice[0], xslice[-1]-xslice[0]))
+
+        collection = BrokenBarHCollection(xranges, [ymin, ymax-ymin], **kwargs)
+        return collection
+
 class RegularPolyCollection(Collection):
     """Draw a collection of regular polygons with *numsides*."""
     _path_generator = mpath.Path.unit_regular_polygon
@@ -619,24 +782,33 @@ class RegularPolyCollection(Collection):
         """
         Collection.__init__(self,**kwargs)
         self._sizes = sizes
+        self._numsides = numsides
         self._paths = [self._path_generator(numsides)]
         self._rotation = rotation
         self.set_transform(transforms.IdentityTransform())
 
     __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
 
+    @allow_rasterization
     def draw(self, renderer):
-        # sizes is the area of the circle circumscribing the polygon
-        # in points^2
         self._transforms = [
             transforms.Affine2D().rotate(-self._rotation).scale(
-                (np.sqrt(x) * renderer.dpi / 72.0) / np.sqrt(np.pi))
+                (np.sqrt(x) * self.figure.dpi / 72.0) / np.sqrt(np.pi))
             for x in self._sizes]
         return Collection.draw(self, renderer)
 
     def get_paths(self):
         return self._paths
 
+    def get_numsides(self):
+        return self._numsides
+
+    def get_rotation(self):
+        return self._rotation
+
+    def get_sizes(self):
+        return self._sizes
+
 
 class StarPolygonCollection(RegularPolyCollection):
     """
@@ -740,6 +912,7 @@ class LineCollection(Collection):
         Collection.__init__(
             self,
             edgecolors=colors,
+            facecolors='none',
             linewidths=linewidths,
             linestyles=linestyles,
             antialiaseds=antialiaseds,
@@ -750,7 +923,6 @@ class LineCollection(Collection):
             pickradius=pickradius,
             **kwargs)
 
-        self._facecolors = np.array([])
         self.set_segments(segments)
 
     def get_paths(self):
@@ -758,10 +930,14 @@ class LineCollection(Collection):
 
     def set_segments(self, segments):
         if segments is None: return
-        segments = [np.asarray(seg, np.float_) for seg in segments]
+        _segments = []
+        for seg in segments:
+            if not np.ma.isMaskedArray(seg):
+                seg = np.asarray(seg, np.float_)
+            _segments.append(seg)
         if self._uniform_offsets is not None:
-            segments = self._add_offsets(segments)
-        self._paths = [mpath.Path(seg) for seg in segments]
+            _segments = self._add_offsets(_segments)
+        self._paths = [mpath.Path(seg) for seg in _segments]
 
     set_verts = set_segments # for compatibility with PolyCollection
 
@@ -783,7 +959,7 @@ class LineCollection(Collection):
         Set the color(s) of the line collection.  *c* can be a
         matplotlib color arg (all patches have same color), or a
         sequence or rgba tuples; if it is a sequence the patches will
-        cycle through the sequence
+        cycle through the sequence.
 
         ACCEPTS: matplotlib color arg or sequence of rgba tuples
         """
@@ -805,6 +981,123 @@ class LineCollection(Collection):
         return self._edgecolors
     get_colors = get_color  # for compatibility with old versions
 
+class CircleCollection(Collection):
+    """
+    A collection of circles, drawn using splines.
+    """
+    def __init__(self, sizes, **kwargs):
+        """
+        *sizes*
+            Gives the area of the circle in points^2
+
+        %(Collection)s
+        """
+        Collection.__init__(self,**kwargs)
+        self._sizes = sizes
+        self.set_transform(transforms.IdentityTransform())
+        self._paths = [mpath.Path.unit_circle()]
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
+    def get_sizes(self):
+        "return sizes of circles"
+        return self._sizes
+
+    def draw(self, renderer):
+        # sizes is the area of the circle circumscribing the polygon
+        # in points^2
+        self._transforms = [
+            transforms.Affine2D().scale(
+                (np.sqrt(x) * self.figure.dpi / 72.0) / np.sqrt(np.pi))
+            for x in self._sizes]
+        return Collection.draw(self, renderer)
+
+    def get_paths(self):
+        return self._paths
+
+class EllipseCollection(Collection):
+    """
+    A collection of ellipses, drawn using splines.
+    """
+    def __init__(self, widths, heights, angles, units='points', **kwargs):
+        """
+        *widths*: sequence
+            half-lengths of first axes (e.g., semi-major axis lengths)
+
+        *heights*: sequence
+            half-lengths of second axes
+
+        *angles*: sequence
+            angles of first axes, degrees CCW from the X-axis
+
+        *units*: ['points' | 'inches' | 'dots' | 'width' | 'height' | 'x' | 'y']
+            units in which majors and minors are given; 'width' and 'height'
+            refer to the dimensions of the axes, while 'x' and 'y'
+            refer to the *offsets* data units.
+
+        Additional kwargs inherited from the base :class:`Collection`:
+
+        %(Collection)s
+        """
+        Collection.__init__(self,**kwargs)
+        self._widths = np.asarray(widths).ravel()
+        self._heights = np.asarray(heights).ravel()
+        self._angles = np.asarray(angles).ravel() *(np.pi/180.0)
+        self._units = units
+        self.set_transform(transforms.IdentityTransform())
+        self._transforms = []
+        self._paths = [mpath.Path.unit_circle()]
+        self._initialized = False
+
+
+    __init__.__doc__ = cbook.dedent(__init__.__doc__) % artist.kwdocd
+
+    def _init(self):
+        def on_dpi_change(fig):
+            self._transforms = []
+        self.figure.callbacks.connect('dpi_changed', on_dpi_change)
+        self._initialized = True
+
+    def set_transforms(self):
+        if not self._initialized:
+            self._init()
+        self._transforms = []
+        ax = self.axes
+        fig = self.figure
+        if self._units in ('x', 'y'):
+            if self._units == 'x':
+                dx0 = ax.viewLim.width
+                dx1 = ax.bbox.width
+            else:
+                dx0 = ax.viewLim.height
+                dx1 = ax.bbox.height
+            sc = dx1/dx0
+        else:
+            if self._units == 'inches':
+                sc = fig.dpi
+            elif self._units == 'points':
+                sc = fig.dpi / 72.0
+            elif self._units == 'width':
+                sc = ax.bbox.width
+            elif self._units == 'height':
+                sc = ax.bbox.height
+            elif self._units == 'dots':
+                sc = 1.0
+            else:
+                raise ValueError('unrecognized units: %s' % self._units)
+
+        _affine = transforms.Affine2D
+        for x, y, a in zip(self._widths, self._heights, self._angles):
+            trans = _affine().scale(x * sc, y * sc).rotate(a)
+            self._transforms.append(trans)
+
+    def draw(self, renderer):
+        if True: ###not self._transforms:
+            self.set_transforms()
+        return Collection.draw(self, renderer)
+
+    def get_paths(self):
+        return self._paths
+
 class PatchCollection(Collection):
     """
     A generic collection of patches.
@@ -840,13 +1133,13 @@ class PatchCollection(Collection):
 
         if match_original:
             def determine_facecolor(patch):
-                if patch.fill():
+                if patch.fill:
                     return patch.get_facecolor()
                 return [0, 0, 0, 0]
 
             facecolors   = [determine_facecolor(p) for p in patches]
             edgecolors   = [p.get_edgecolor() for p in patches]
-            linewidths   = [p.get_linewidths() for p in patches]
+            linewidths   = [p.get_linewidth() for p in patches]
             antialiaseds = [p.get_antialiased() for p in patches]
 
             Collection.__init__(
@@ -870,6 +1163,6 @@ class PatchCollection(Collection):
 
 artist.kwdocd['Collection'] = patchstr = artist.kwdoc(Collection)
 for k in ('QuadMesh', 'PolyCollection', 'BrokenBarHCollection', 'RegularPolyCollection',
-          'StarPolygonCollection'):
+          'StarPolygonCollection', 'PatchCollection', 'CircleCollection'):
     artist.kwdocd[k] = patchstr
 artist.kwdocd['LineCollection'] = artist.kwdoc(LineCollection)
