@@ -51,7 +51,7 @@ basedir = {
     'linux'  : ['/usr/local', '/usr',],
     'cygwin' : ['/usr/local', '/usr',],
     'darwin' : ['/sw/lib/freetype2', '/sw/lib/freetype219', '/usr/local',
-                '/usr', '/sw', '/usr/X11R6'],
+                '/usr', '/sw'],
     'freebsd4' : ['/usr/local', '/usr'],
     'freebsd5' : ['/usr/local', '/usr'],
     'freebsd6' : ['/usr/local', '/usr'],
@@ -106,8 +106,7 @@ options = {'display_status': True,
            'build_macosx': 'auto',
            'build_image': True,
            'build_windowing': True,
-           'backend': None,
-           'numerix': None}
+           'backend': None}
 
 # Based on the contents of setup.cfg, determine the build options
 if os.path.exists("setup.cfg"):
@@ -127,7 +126,6 @@ if os.path.exists("setup.cfg"):
                                                          "dateutil")
     except: options['provide_dateutil'] = 'auto'
 
-
     try: options['build_gtk'] = config.getboolean("gui_support", "gtk")
     except: options['build_gtk'] = 'auto'
 
@@ -140,10 +138,10 @@ if os.path.exists("setup.cfg"):
     try: options['build_wxagg'] = config.getboolean("gui_support", "wxagg")
     except: options['build_wxagg'] = 'auto'
 
-    try: options['backend'] = config.get("rc_options", "backend")
-    except: pass
+    try: options['build_macosx'] = config.getboolean("gui_support", "macosx")
+    except: options['build_macosx'] = 'auto'
 
-    try: options['numerix'] = config.get("rc_options", "numerix")
+    try: options['backend'] = config.get("rc_options", "backend")
     except: pass
 
 
@@ -176,7 +174,7 @@ def run_child_process(cmd):
                          stdin=subprocess.PIPE,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT,
-                         close_fds=True)
+                         close_fds=(sys.platform != 'win32'))
     return p.stdin, p.stdout
 
 class CleanUpFile:
@@ -434,7 +432,6 @@ def check_provide_dateutil(hasdatetime=True):
             print_status("dateutil", "present, version unknown")
             return False
 
-
 def check_for_dvipng():
     try:
         stdin, stdout = run_child_process('dvipng -version')
@@ -461,7 +458,7 @@ def check_for_latex():
     try:
         stdin, stdout = run_child_process('latex -version')
         line = stdout.readlines()[0]
-        pattern = '3\.1\d+'
+        pattern = '(3\.1\d+)|(MiKTeX \d+.\d+)'
         match = re.search(pattern, line)
         print_status("latex", match.group(0))
         return True
@@ -548,10 +545,7 @@ def add_ft2font_flags(module):
     else:
         add_base_flags(module)
         module.libraries.append('z')
-
-    if sys.platform == 'win32' and win32_compiler == 'mingw32':
-        module.libraries.append('gw32c')
-
+    
     # put this last for library link order
     module.libraries.extend(std_libs)
 
@@ -601,6 +595,13 @@ def check_for_gtk():
     if explanation is not None:
         print_message(explanation)
 
+    # Switch off the event loop for PyGTK >= 2.15.0
+    if gotit:
+        try:
+            gtk.set_interactive(False)
+        except AttributeError: # PyGTK < 2.15.0
+            pass
+
     return gotit
 
 def add_pygtk_flags(module):
@@ -848,6 +849,7 @@ def query_tcltk():
         tk.withdraw()
         tcl_lib_dir = str(tk.getvar('tcl_library'))
         tk_lib_dir = str(tk.getvar('tk_library'))
+        tk.destroy()
 
     # Save directories and version string to cache
     TCL_TK_CACHE = tcl_lib_dir, tk_lib_dir, str(Tkinter.TkVersion)[:3]
@@ -958,11 +960,14 @@ def add_tk_flags(module):
     message = None
     if sys.platform == 'win32':
         major, minor1, minor2, s, tmp = sys.version_info
-        if major == 2 and minor1 in [3, 4, 5]:
-            module.include_dirs.extend(['win32_static/include/tcl8.4'])
+        if major == 2 and minor1 == 6:
+            module.include_dirs.extend(['win32_static/include/tcl85'])
+            module.libraries.extend(['tk85', 'tcl85'])
+        elif major == 2 and minor1 in [3, 4, 5]:
+            module.include_dirs.extend(['win32_static/include/tcl84'])
             module.libraries.extend(['tk84', 'tcl84'])
         elif major == 2 and minor1 == 2:
-            module.include_dirs.extend(['win32_static/include/tcl8.3'])
+            module.include_dirs.extend(['win32_static/include/tcl83'])
             module.libraries.extend(['tk83', 'tcl83'])
         else:
             raise RuntimeError('No tk/win32 support for this python version yet')
@@ -1068,8 +1073,7 @@ def build_windowing(ext_modules, packages):
     global BUILT_WINDOWING
     if BUILT_WINDOWING: return # only build it if you you haven't already
     module = Extension('matplotlib._windowing',
-                       ['src/_windowing.cpp',
-                        ],
+                       ['src/_windowing.cpp'],
                        )
     add_windowing_flags(module)
     ext_modules.append(module)
@@ -1082,7 +1086,8 @@ def build_ft2font(ext_modules, packages):
     deps.extend(glob.glob('CXX/*.cxx'))
     deps.extend(glob.glob('CXX/*.c'))
 
-    module = Extension('matplotlib.ft2font', deps)
+    module = Extension('matplotlib.ft2font', deps,
+                       define_macros=[('PY_ARRAYAUNIQUE_SYMBOL', 'MPL_ARRAY_API')])
     add_ft2font_flags(module)
     ext_modules.append(module)
     BUILT_FT2FONT = True
@@ -1103,12 +1108,13 @@ def build_ttconv(ext_modules, packages):
 def build_gtkagg(ext_modules, packages):
     global BUILT_GTKAGG
     if BUILT_GTKAGG: return # only build it if you you haven't already
-    deps = ['src/_gtkagg.cpp', 'src/mplutils.cpp']#, 'src/_transforms.cpp']
+    deps = ['src/agg_py_transforms.cpp', 'src/_gtkagg.cpp', 'src/mplutils.cpp']
     deps.extend(glob.glob('CXX/*.cxx'))
     deps.extend(glob.glob('CXX/*.c'))
 
     module = Extension('matplotlib.backends._gtkagg',
                        deps,
+                       define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
                        )
 
     # add agg flags before pygtk because agg only supports freetype1
@@ -1125,12 +1131,13 @@ def build_gtkagg(ext_modules, packages):
 def build_tkagg(ext_modules, packages):
     global BUILT_TKAGG
     if BUILT_TKAGG: return # only build it if you you haven't already
-    deps = ['src/_tkagg.cpp']
+    deps = ['src/agg_py_transforms.cpp', 'src/_tkagg.cpp']
     deps.extend(glob.glob('CXX/*.cxx'))
     deps.extend(glob.glob('CXX/*.c'))
 
     module = Extension('matplotlib.backends._tkagg',
                        deps,
+                       define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
                        )
 
     add_tk_flags(module) # do this first
@@ -1160,19 +1167,26 @@ def build_wxagg(ext_modules, packages):
      ext_modules.append(module)
      BUILT_WXAGG = True
 
-
 def build_macosx(ext_modules, packages):
     global BUILT_MACOSX
     if BUILT_MACOSX: return # only build it if you you haven't already
+    deps = ['src/_macosx.m',
+            'CXX/cxx_extensions.cxx',
+            'CXX/cxxextensions.c',
+            'CXX/cxxsupport.cxx',
+            'CXX/IndirectPythonInterface.cxx',
+            'src/agg_py_transforms.cpp',
+            'src/path_cleanup.cpp']
     module = Extension('matplotlib.backends._macosx',
-                       ['src/_macosx.m'],
+                       deps,
                        extra_link_args = ['-framework','Cocoa'],
+                       define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
                       )
     add_numpy_flags(module)
+    add_agg_flags(module)
     ext_modules.append(module)
     BUILT_MACOSX = True
 
-
 def build_png(ext_modules, packages):
     global BUILT_PNG
     if BUILT_PNG: return # only build it if you you haven't already
@@ -1185,6 +1199,7 @@ def build_png(ext_modules, packages):
         'matplotlib._png',
         deps,
         include_dirs=numpy_inc_dirs,
+        define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
         )
 
     add_png_flags(module)
@@ -1197,7 +1212,6 @@ def build_agg(ext_modules, packages):
     global BUILT_AGG
     if BUILT_AGG: return # only build it if you you haven't already
 
-
     agg = (
            'agg_trans_affine.cpp',
            'agg_bezier_arc.cpp',
@@ -1207,22 +1221,20 @@ def build_agg(ext_modules, packages):
            'agg_image_filters.cpp',
            )
 
-
     deps = ['%s/src/%s'%(AGG_VERSION, name) for name in agg]
-    deps.extend(('src/_image.cpp', 'src/ft2font.cpp', 'src/mplutils.cpp'))
+    deps.extend(['src/mplutils.cpp', 'src/agg_py_transforms.cpp'])
     deps.extend(glob.glob('CXX/*.cxx'))
     deps.extend(glob.glob('CXX/*.c'))
-
     temp_copy('src/_backend_agg.cpp', 'src/backend_agg.cpp')
     deps.append('src/backend_agg.cpp')
     module = Extension(
         'matplotlib.backends._backend_agg',
         deps,
         include_dirs=numpy_inc_dirs,
+        define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
         )
 
     add_numpy_flags(module)
-
     add_agg_flags(module)
     add_ft2font_flags(module)
     ext_modules.append(module)
@@ -1245,11 +1257,14 @@ def build_path(ext_modules, packages):
     deps.extend(glob.glob('CXX/*.c'))
 
     temp_copy('src/_path.cpp', 'src/path.cpp')
-    deps.extend(['src/path.cpp'])
+    deps.extend(['src/agg_py_transforms.cpp',
+                 'src/path_cleanup.cpp',
+                 'src/path.cpp'])
     module = Extension(
         'matplotlib._path',
         deps,
         include_dirs=numpy_inc_dirs,
+        define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
         )
 
     add_numpy_flags(module)
@@ -1278,6 +1293,7 @@ def build_image(ext_modules, packages):
         'matplotlib._image',
         deps,
         include_dirs=numpy_inc_dirs,
+        define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
         )
 
     add_numpy_flags(module)
@@ -1297,7 +1313,9 @@ def build_delaunay(ext_modules, packages):
                  "delaunay_utils.cpp", "natneighbors.cpp"]
     sourcefiles = [os.path.join('lib/matplotlib/delaunay',s) for s in sourcefiles]
     delaunay = Extension('matplotlib._delaunay',sourcefiles,
-                         include_dirs=numpy_inc_dirs)
+                         include_dirs=numpy_inc_dirs,
+                         define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
+                         )
     add_numpy_flags(delaunay)
     add_base_flags(delaunay)
     ext_modules.append(delaunay)
@@ -1313,6 +1331,7 @@ def build_contour(ext_modules, packages):
         'matplotlib._cntr',
         [ 'src/cntr.c'],
         include_dirs=numpy_inc_dirs,
+        define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
         )
     add_numpy_flags(module)
     add_base_flags(module)
@@ -1328,6 +1347,7 @@ def build_nxutils(ext_modules, packages):
         'matplotlib.nxutils',
         [ 'src/nxutils.c'],
         include_dirs=numpy_inc_dirs,
+        define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
         )
     add_numpy_flags(module)
     add_base_flags(module)
@@ -1343,9 +1363,10 @@ def build_gdk(ext_modules, packages):
     temp_copy('src/_backend_gdk.c', 'src/backend_gdk.c')
     module = Extension(
         'matplotlib.backends._backend_gdk',
-        ['src/backend_gdk.c', ],
+        ['src/backend_gdk.c'],
         libraries = [],
         include_dirs=numpy_inc_dirs,
+        define_macros=[('PY_ARRAY_UNIQUE_SYMBOL', 'MPL_ARRAY_API')]
         )
 
     add_numpy_flags(module)
